/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package eu.agrosense.client.calendar;

import eu.agrosense.client.calendar.api.gui.CalendarComponent;
import eu.agrosense.client.calendar.api.model.CalendarNode;
import java.awt.BorderLayout;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDropEvent;
import javafx.application.Platform;
import javafx.embed.swing.JFXPanel;
import javafx.scene.Scene;
import javafx.scene.layout.BorderPane;
import nl.cloudfarming.client.model.Temporal;
import org.joda.time.LocalDate;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.nodes.Node;
import org.openide.nodes.NodeTransfer;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;

@TopComponent.Description(preferredID = "CalendarViewerTopComponent", iconBase = "nl/cloudfarming/client/icon/calendar.png", persistenceType = TopComponent.PERSISTENCE_ALWAYS)
@TopComponent.Registration(mode = "calendar", openAtStartup = false)
@ActionID(category = "Window", id = "eu.agrosense.client.calendar.CalendarViewerTopComponent")
@ActionReference(path = "Menu/Window")
@TopComponent.OpenActionRegistration(displayName = "#CTL_CalendarViewerAction", preferredID = "CalendarViewerTopComponent")
@NbBundle.Messages({
    "CTL_CalendarViewerAction=Calendar",
    "CTL_CalendarViewerTopComponent=Calendar Window",
    "HINT_CalendarViewerTopComponent=This is a Calendar window"
})
public class CalendarViewerTopComponent extends TopComponent {

    private static CalendarViewerTopComponent instance;
    private static final String ICON_PATH = "nl/cloudfarming/client/icon/calendar32.png";
    private static final String PREFERRED_ID = "CalendarViewerTopComponent";
    private JFXPanel fxPanel;
    private CalendarDropTarget calendarDropTarget;
    
    public CalendarViewerTopComponent() {
        setIcon(ImageUtilities.loadImage(ICON_PATH, true));
        setName(NbBundle.getMessage(CalendarViewerTopComponent.class, "CTL_CalendarViewerTopComponent"));
        initComponents();
    }

    private void initComponents() {
        setLayout(new BorderLayout());
        fxPanel = new JFXPanel();
        calendarDropTarget = new CalendarDropTarget();

        Platform.runLater(new Runnable() {
            @Override
            public void run() {
                initFX(fxPanel);
            }
        });

        fxPanel.setDropTarget(calendarDropTarget);
        add(fxPanel, BorderLayout.CENTER);
    }

    protected void initFX(JFXPanel fxPanel) {
        Scene scene = createScene();
        fxPanel.setScene(scene);
    }

    private Scene createScene() {
        BorderPane borderPane = new BorderPane();
        Scene scene = new Scene(borderPane);

        CalendarComponent calendarComponent = new CalendarComponent();
        calendarDropTarget.setCalendarComponent(calendarComponent);
        scene.getStylesheets().add(CalendarViewerTopComponent.class.getResource("chart-style.css").toExternalForm());
        borderPane.setCenter(calendarComponent);
        return scene;
    }

    public static synchronized CalendarViewerTopComponent getDefault() {
        if (instance == null) {
            instance = new CalendarViewerTopComponent();
        }
        return instance;
    }

    public static synchronized CalendarViewerTopComponent findInstance() {
        TopComponent win = WindowManager.getDefault().findTopComponent(PREFERRED_ID);
        if (win == null) {
            return getDefault();
        }
        if (win instanceof CalendarViewerTopComponent) {
            return (CalendarViewerTopComponent) win;
        }
        return getDefault();
    }

    @Override
    public int getPersistenceType() {
        return TopComponent.PERSISTENCE_ALWAYS;
    }

    void writeProperties(java.util.Properties p) {
    }

    Object readProperties(java.util.Properties p) {
        if (instance == null) {
            instance = this;
        }
        instance.readPropertiesImpl(p);
        return instance;
    }

    private void readPropertiesImpl(java.util.Properties p) {
    }
   

    private class CalendarDropTarget extends DropTarget {

        private CalendarComponent calendarComponent;
        
        public void setCalendarComponent(CalendarComponent calendarComponent) {
            this.calendarComponent = calendarComponent;
        }
        
        private void handleDrop(Node node) {
            final Temporal temporal = node.getLookup().lookup(Temporal.class);
            
            if (temporal != null) {
                final CalendarNode fieldNode = new CalendarNode(temporal.getName(), new LocalDate(temporal.getStartDate()), new LocalDate(temporal.getEndDate()));
                for (Temporal child : temporal.getTemporalChildren()) {
                    CalendarNode childModel = new CalendarNode(child.getName(), new LocalDate(child.getStartDate()), new LocalDate(child.getEndDate()));
                    fieldNode.addChild(childModel);
                }
                    
                Platform.runLater(new Runnable() {
                    @Override
                    public void run() {
                        calendarComponent.addNode(fieldNode, temporal.getName());
                    }
                });
            }
            
        }

        @Override
        public synchronized void drop(DropTargetDropEvent dtde) {

            //right now we accept all nodes
            Node node = NodeTransfer.node(dtde.getTransferable(), NodeTransfer.DND_COPY);
            if (node != null) {
                handleDrop(node);
                dtde.acceptDrop(DnDConstants.ACTION_COPY);
                return;
            }

            Node[] nodes = NodeTransfer.nodes(dtde.getTransferable(), NodeTransfer.DND_COPY);
            if (nodes != null) {
                for (Node n : nodes) {
                    handleDrop(n);
                }
                dtde.acceptDrop(DnDConstants.ACTION_COPY);
                return;
            }
            
            dtde.rejectDrop();
            dtde.dropComplete(false);
        }
    }
}
